// Generated by CoffeeScript 1.10.0
(function() {
  var Script, ShapingPlan;

  Script = require('../layout/Script');

  ShapingPlan = (function() {
    function ShapingPlan(font, script, language) {
      this.font = font;
      this.script = script;
      this.language = language;
      this.direction = Script.direction(this.script);
      this.stages = [];
      this.globalFeatures = {};
      this.allFeatures = {};
    }

    ShapingPlan.prototype._addFeatures = function(features) {
      var feature, i, len, stage;
      stage = this.stages[this.stages.length - 1];
      for (i = 0, len = features.length; i < len; i++) {
        feature = features[i];
        if (!this.allFeatures[feature]) {
          stage.push(feature);
          this.allFeatures[feature] = true;
        }
      }
    };

    ShapingPlan.prototype._addGlobal = function(features) {
      var feature, i, len;
      for (i = 0, len = features.length; i < len; i++) {
        feature = features[i];
        this.globalFeatures[feature] = true;
      }
    };

    ShapingPlan.prototype.add = function(arg, global) {
      var features;
      if (global == null) {
        global = true;
      }
      if (this.stages.length === 0) {
        this.stages.push([]);
      }
      if (typeof arg === 'string') {
        arg = [arg];
      }
      if (Array.isArray(arg)) {
        this._addFeatures(arg);
        if (global) {
          return this._addGlobal(arg);
        }
      } else if (typeof arg === 'object') {
        features = (arg.global || []).concat(arg.local || []);
        this._addFeatures(features);
        if (arg.global) {
          return this._addGlobal(arg.global);
        }
      } else {
        throw new Error("Unsupported argument to ShapingPlan#add");
      }
    };

    ShapingPlan.prototype.addStage = function(arg, global) {
      if (typeof arg === 'function') {
        return this.stages.push(arg, []);
      } else {
        this.stages.push([]);
        return this.add(arg, global);
      }
    };

    ShapingPlan.prototype.assignGlobalFeatures = function(glyphs) {
      var feature, glyph, i, len;
      for (i = 0, len = glyphs.length; i < len; i++) {
        glyph = glyphs[i];
        for (feature in this.globalFeatures) {
          glyph.features[feature] = true;
        }
      }
    };

    ShapingPlan.prototype.process = function(processor, glyphs, positions) {
      var i, item, len, ref;
      processor.selectScript(this.script, this.language);
      ref = this.stages;
      for (i = 0, len = ref.length; i < len; i++) {
        item = ref[i];
        if (typeof item === 'function') {
          item(glyphs, positions);
        } else if (item.length > 0) {
          processor.applyFeatures(item, glyphs, positions);
        }
      }
    };

    return ShapingPlan;

  })();

  module.exports = ShapingPlan;

}).call(this);
